import "dart:async";
import 'dart:convert';
import 'package:chopper/chopper.dart';

import 'package:http/http.dart' show MultipartFile;

part "test_service.chopper.dart";

@ChopperApi(baseUrl: "/test")
abstract class HttpTestService extends ChopperService {
  static HttpTestService create([ChopperClient client]) =>
      _$HttpTestService(client);

  @Get(path: "get/{id}")
  Future<Response> getTest(
    @Path() String id, {
    @Header('test') String dynamicHeader,
  });

  @Get(path: "get")
  Future<Response<Stream<List<int>>>> getStreamTest();

  @Get(path: "query")
  Future<Response> getQueryTest({
    @Query('name') String name,
    @Query('int') int number,
    @Query('default_value') int def = 42,
  });

  @Get(path: "query_map")
  Future<Response> getQueryMapTest(@QueryMap() Map<String, dynamic> query);

  @Get(path: "query_map")
  Future<Response> getQueryMapTest2(
    @QueryMap() Map<String, dynamic> query, {
    @Query('test') bool test,
  });

  @Post(path: "post")
  Future<Response> postTest(@Body() String data);

  @Post(path: "post")
  Future<Response> postStreamTest(@Body() Stream<List<int>> byteStream);

  @Put(path: 'put/{id}')
  Future<Response> putTest(@Path('id') String test, @Body() String data);

  @Delete(path: 'delete/{id}', headers: {'foo': 'bar'})
  Future<Response> deleteTest(@Path() String id);

  @Patch(path: 'patch/{id}')
  Future<Response> patchTest(@Path() String id, @Body() String data);

  @Post(path: 'map')
  Future<Response> mapTest(@Body() Map map);

  @Post(path: 'map/json')
  @FactoryConverter(
      request: customConvertRequest, response: customConvertResponse)
  Future<Response> forceJsonTest(@Body() Map map);

  @Post(path: 'multi')
  @multipart
  Future<Response> postResources(
    @Part('1') Map a,
    @Part('2') Map b,
  );

  @Post(path: 'file')
  @multipart
  Future<Response> postFile(
    @PartFile('file') List<int> bytes,
  );

  @Post(path: 'file')
  @multipart
  Future<Response> postMultipartFile(
    @PartFile() MultipartFile file, {
    @Part() String id,
  });

  @Get(path: 'https://test.com')
  Future fullUrl();

  @Get(path: '/list/string')
  Future<Response<List<String>>> listString();
}

Request customConvertRequest(Request req) {
  final r = JsonConverter().convertRequest(req);
  return applyHeader(r, 'customConverter', 'true');
}

Response<T> customConvertResponse<T>(Response res) =>
    res.replace(body: json.decode(res.body));
